#ifndef OPCUA_STATUSCODES_H_
#define OPCUA_STATUSCODES_H_

#include "ua_types.h"

static const UA_StatusCode UA_STATUSCODE_GOOD = 0x00;
static const UA_StatusCode UA_STATUSCODE_BADUNEXPECTEDERROR = 0x80010000; // An unexpected error occurred.
static const UA_StatusCode UA_STATUSCODE_BADINTERNALERROR = 0x80020000; // An internal error occurred as a result of a programming or configuration error.
static const UA_StatusCode UA_STATUSCODE_BADOUTOFMEMORY = 0x80030000; // Not enough memory to complete the operation.
static const UA_StatusCode UA_STATUSCODE_BADRESOURCEUNAVAILABLE = 0x80040000; // An operating system resource is not available.
static const UA_StatusCode UA_STATUSCODE_BADCOMMUNICATIONERROR = 0x80050000; // A low level communication error occurred.
static const UA_StatusCode UA_STATUSCODE_BADENCODINGERROR = 0x80060000; // Encoding halted because of invalid data in the objects being serialized.
static const UA_StatusCode UA_STATUSCODE_BADDECODINGERROR = 0x80070000; // Decoding halted because of invalid data in the stream.
static const UA_StatusCode UA_STATUSCODE_BADENCODINGLIMITSEXCEEDED = 0x80080000; // The message encoding/decoding limits imposed by the stack have been exceeded.
static const UA_StatusCode UA_STATUSCODE_BADREQUESTTOOLARGE = 0x80b80000; // The request message size exceeds limits set by the server.
static const UA_StatusCode UA_STATUSCODE_BADRESPONSETOOLARGE = 0x80b90000; // The response message size exceeds limits set by the client.
static const UA_StatusCode UA_STATUSCODE_BADUNKNOWNRESPONSE = 0x80090000; // An unrecognized response was received from the server.
static const UA_StatusCode UA_STATUSCODE_BADTIMEOUT = 0x800a0000; // The operation timed out.
static const UA_StatusCode UA_STATUSCODE_BADSERVICEUNSUPPORTED = 0x800b0000; // The server does not support the requested service.
static const UA_StatusCode UA_STATUSCODE_BADSHUTDOWN = 0x800c0000; // The operation was cancelled because the application is shutting down.
static const UA_StatusCode UA_STATUSCODE_BADSERVERNOTCONNECTED = 0x800d0000; // The operation could not complete because the client is not connected to the server.
static const UA_StatusCode UA_STATUSCODE_BADSERVERHALTED = 0x800e0000; // The server has stopped and cannot process any requests.
static const UA_StatusCode UA_STATUSCODE_BADNOTHINGTODO = 0x800f0000; // There was nothing to do because the client passed a list of operations with no elements.
static const UA_StatusCode UA_STATUSCODE_BADTOOMANYOPERATIONS = 0x80100000; // The request could not be processed because it specified too many operations.
static const UA_StatusCode UA_STATUSCODE_BADTOOMANYMONITOREDITEMS = 0x80db0000; // The request could not be processed because there are too many monitored items in the subscription.
static const UA_StatusCode UA_STATUSCODE_BADDATATYPEIDUNKNOWN = 0x80110000; // The extension object cannot be (de)serialized because the data type id is not recognized.
static const UA_StatusCode UA_STATUSCODE_BADCERTIFICATEINVALID = 0x80120000; // The certificate provided as a parameter is not valid.
static const UA_StatusCode UA_STATUSCODE_BADSECURITYCHECKSFAILED = 0x80130000; // An error occurred verifying security.
static const UA_StatusCode UA_STATUSCODE_BADCERTIFICATETIMEINVALID = 0x80140000; // The Certificate has expired or is not yet valid.
static const UA_StatusCode UA_STATUSCODE_BADCERTIFICATEISSUERTIMEINVALID = 0x80150000; // An Issuer Certificate has expired or is not yet valid.
static const UA_StatusCode UA_STATUSCODE_BADCERTIFICATEHOSTNAMEINVALID = 0x80160000; // The HostName used to connect to a Server does not match a HostName in the Certificate.
static const UA_StatusCode UA_STATUSCODE_BADCERTIFICATEURIINVALID = 0x80170000; // The URI specified in the ApplicationDescription does not match the URI in the Certificate.
static const UA_StatusCode UA_STATUSCODE_BADCERTIFICATEUSENOTALLOWED = 0x80180000; // The Certificate may not be used for the requested operation.
static const UA_StatusCode UA_STATUSCODE_BADCERTIFICATEISSUERUSENOTALLOWED = 0x80190000; // The Issuer Certificate may not be used for the requested operation.
static const UA_StatusCode UA_STATUSCODE_BADCERTIFICATEUNTRUSTED = 0x801a0000; // The Certificate is not trusted.
static const UA_StatusCode UA_STATUSCODE_BADCERTIFICATEREVOCATIONUNKNOWN = 0x801b0000; // It was not possible to determine if the Certificate has been revoked.
static const UA_StatusCode UA_STATUSCODE_BADCERTIFICATEISSUERREVOCATIONUNKNOWN = 0x801c0000; // It was not possible to determine if the Issuer Certificate has been revoked.
static const UA_StatusCode UA_STATUSCODE_BADCERTIFICATEREVOKED = 0x801d0000; // The Certificate has been revoked.
static const UA_StatusCode UA_STATUSCODE_BADCERTIFICATEISSUERREVOKED = 0x801e0000; // The Issuer Certificate has been revoked.
static const UA_StatusCode UA_STATUSCODE_BADUSERACCESSDENIED = 0x801f0000; // User does not have permission to perform the requested operation.
static const UA_StatusCode UA_STATUSCODE_BADIDENTITYTOKENINVALID = 0x80200000; // The user identity token is not valid.
static const UA_StatusCode UA_STATUSCODE_BADIDENTITYTOKENREJECTED = 0x80210000; // The user identity token is valid but the server has rejected it.
static const UA_StatusCode UA_STATUSCODE_BADSECURECHANNELIDINVALID = 0x80220000; // The specified secure channel is no longer valid.
static const UA_StatusCode UA_STATUSCODE_BADINVALIDTIMESTAMP = 0x80230000; // The timestamp is outside the range allowed by the server.
static const UA_StatusCode UA_STATUSCODE_BADNONCEINVALID = 0x80240000; // The nonce does appear to be not a random value or it is not the correct length.
static const UA_StatusCode UA_STATUSCODE_BADSESSIONIDINVALID = 0x80250000; // The session id is not valid.
static const UA_StatusCode UA_STATUSCODE_BADSESSIONCLOSED = 0x80260000; // The session was closed by the client.
static const UA_StatusCode UA_STATUSCODE_BADSESSIONNOTACTIVATED = 0x80270000; // The session cannot be used because ActivateSession has not been called.
static const UA_StatusCode UA_STATUSCODE_BADSUBSCRIPTIONIDINVALID = 0x80280000; // The subscription id is not valid.
static const UA_StatusCode UA_STATUSCODE_BADREQUESTHEADERINVALID = 0x802a0000; // The header for the request is missing or invalid.
static const UA_StatusCode UA_STATUSCODE_BADTIMESTAMPSTORETURNINVALID = 0x802b0000; // The timestamps to return parameter is invalid.
static const UA_StatusCode UA_STATUSCODE_BADREQUESTCANCELLEDBYCLIENT = 0x802c0000; // The request was cancelled by the client.
static const UA_StatusCode UA_STATUSCODE_GOODSUBSCRIPTIONTRANSFERRED = 0x002d0000; // The subscription was transferred to another session.
static const UA_StatusCode UA_STATUSCODE_GOODCOMPLETESASYNCHRONOUSLY = 0x002e0000; // The processing will complete asynchronously.
static const UA_StatusCode UA_STATUSCODE_GOODOVERLOAD = 0x002f0000; // Sampling has slowed down due to resource limitations.
static const UA_StatusCode UA_STATUSCODE_GOODCLAMPED = 0x00300000; // The value written was accepted but was clamped.
static const UA_StatusCode UA_STATUSCODE_BADNOCOMMUNICATION = 0x80310000; // Communication with the data source is defined, but not established, and there is no last known value available.
static const UA_StatusCode UA_STATUSCODE_BADWAITINGFORINITIALDATA = 0x80320000; // Waiting for the server to obtain values from the underlying data source.
static const UA_StatusCode UA_STATUSCODE_BADNODEIDINVALID = 0x80330000; // The syntax of the node id is not valid.
static const UA_StatusCode UA_STATUSCODE_BADNODEIDUNKNOWN = 0x80340000; // The node id refers to a node that does not exist in the server address space.
static const UA_StatusCode UA_STATUSCODE_BADATTRIBUTEIDINVALID = 0x80350000; // The attribute is not supported for the specified Node.
static const UA_StatusCode UA_STATUSCODE_BADINDEXRANGEINVALID = 0x80360000; // The syntax of the index range parameter is invalid.
static const UA_StatusCode UA_STATUSCODE_BADINDEXRANGENODATA = 0x80370000; // No data exists within the range of indexes specified.
static const UA_StatusCode UA_STATUSCODE_BADDATAENCODINGINVALID = 0x80380000; // The data encoding is invalid.
static const UA_StatusCode UA_STATUSCODE_BADDATAENCODINGUNSUPPORTED = 0x80390000; // The server does not support the requested data encoding for the node.
static const UA_StatusCode UA_STATUSCODE_BADNOTREADABLE = 0x803a0000; // The access level does not allow reading or subscribing to the Node.
static const UA_StatusCode UA_STATUSCODE_BADNOTWRITABLE = 0x803b0000; // The access level does not allow writing to the Node.
static const UA_StatusCode UA_STATUSCODE_BADOUTOFRANGE = 0x803c0000; // The value was out of range.
static const UA_StatusCode UA_STATUSCODE_BADNOTSUPPORTED = 0x803d0000; // The requested operation is not supported.
static const UA_StatusCode UA_STATUSCODE_BADNOTFOUND = 0x803e0000; // A requested item was not found or a search operation ended without success.
static const UA_StatusCode UA_STATUSCODE_BADOBJECTDELETED = 0x803f0000; // The object cannot be used because it has been deleted.
static const UA_StatusCode UA_STATUSCODE_BADNOTIMPLEMENTED = 0x80400000; // Requested operation is not implemented.
static const UA_StatusCode UA_STATUSCODE_BADMONITORINGMODEINVALID = 0x80410000; // The monitoring mode is invalid.
static const UA_StatusCode UA_STATUSCODE_BADMONITOREDITEMIDINVALID = 0x80420000; // The monitoring item id does not refer to a valid monitored item.
static const UA_StatusCode UA_STATUSCODE_BADMONITOREDITEMFILTERINVALID = 0x80430000; // The monitored item filter parameter is not valid.
static const UA_StatusCode UA_STATUSCODE_BADMONITOREDITEMFILTERUNSUPPORTED = 0x80440000; // The server does not support the requested monitored item filter.
static const UA_StatusCode UA_STATUSCODE_BADFILTERNOTALLOWED = 0x80450000; // A monitoring filter cannot be used in combination with the attribute specified.
static const UA_StatusCode UA_STATUSCODE_BADSTRUCTUREMISSING = 0x80460000; // A mandatory structured parameter was missing or null.
static const UA_StatusCode UA_STATUSCODE_BADEVENTFILTERINVALID = 0x80470000; // The event filter is not valid.
static const UA_StatusCode UA_STATUSCODE_BADCONTENTFILTERINVALID = 0x80480000; // The content filter is not valid.
static const UA_StatusCode UA_STATUSCODE_BADFILTEROPERATORINVALID = 0x80c10000; // An unregognized operator was provided in a filter.
static const UA_StatusCode UA_STATUSCODE_BADFILTEROPERATORUNSUPPORTED = 0x80c20000; // A valid operator was provided, but the server does not provide support for this filter operator.
static const UA_StatusCode UA_STATUSCODE_BADFILTEROPERANDCOUNTMISMATCH = 0x80c30000; // The number of operands provided for the filter operator was less then expected for the operand provided.
static const UA_StatusCode UA_STATUSCODE_BADFILTEROPERANDINVALID = 0x80490000; // The operand used in a content filter is not valid.
static const UA_StatusCode UA_STATUSCODE_BADFILTERELEMENTINVALID = 0x80c40000; // The referenced element is not a valid element in the content filter.
static const UA_StatusCode UA_STATUSCODE_BADFILTERLITERALINVALID = 0x80c50000; // The referenced literal is not a valid value.
static const UA_StatusCode UA_STATUSCODE_BADCONTINUATIONPOINTINVALID = 0x804a0000; // The continuation point provide is longer valid.
static const UA_StatusCode UA_STATUSCODE_BADNOCONTINUATIONPOINTS = 0x804b0000; // The operation could not be processed because all continuation points have been allocated.
static const UA_StatusCode UA_STATUSCODE_BADREFERENCETYPEIDINVALID = 0x804c0000; // The operation could not be processed because all continuation points have been allocated.
static const UA_StatusCode UA_STATUSCODE_BADBROWSEDIRECTIONINVALID = 0x804d0000; // The browse direction is not valid.
static const UA_StatusCode UA_STATUSCODE_BADNODENOTINVIEW = 0x804e0000; // The node is not part of the view.
static const UA_StatusCode UA_STATUSCODE_BADSERVERURIINVALID = 0x804f0000; // The ServerUri is not a valid URI.
static const UA_StatusCode UA_STATUSCODE_BADSERVERNAMEMISSING = 0x80500000; // No ServerName was specified.
static const UA_StatusCode UA_STATUSCODE_BADDISCOVERYURLMISSING = 0x80510000; // No DiscoveryUrl was specified.
static const UA_StatusCode UA_STATUSCODE_BADSEMPAHOREFILEMISSING = 0x80520000; // The semaphore file specified by the client is not valid.
static const UA_StatusCode UA_STATUSCODE_BADREQUESTTYPEINVALID = 0x80530000; // The security token request type is not valid.
static const UA_StatusCode UA_STATUSCODE_BADSECURITYMODEREJECTED = 0x80540000; // The security mode does not meet the requirements set by the Server.
static const UA_StatusCode UA_STATUSCODE_BADSECURITYPOLICYREJECTED = 0x80550000; // The security policy does not meet the requirements set by the Server.
static const UA_StatusCode UA_STATUSCODE_BADTOOMANYSESSIONS = 0x80560000; // The server has reached its maximum number of sessions.
static const UA_StatusCode UA_STATUSCODE_BADUSERSIGNATUREINVALID = 0x80570000; // The user token signature is missing or invalid.
static const UA_StatusCode UA_STATUSCODE_BADAPPLICATIONSIGNATUREINVALID = 0x80580000; // The signature generated with the client certificate is missing or invalid.
static const UA_StatusCode UA_STATUSCODE_BADNOVALIDCERTIFICATES = 0x80590000; // The client did not provide at least one software certificate that is valid and meets the profile requirements for the server.
static const UA_StatusCode UA_STATUSCODE_BADIDENTITYCHANGENOTSUPPORTED = 0x80c60000; // The Server does not support changing the user identity assigned to the session.
static const UA_StatusCode UA_STATUSCODE_BADREQUESTCANCELLEDBYREQUEST = 0x805a0000; // The request was cancelled by the client with the Cancel service.
static const UA_StatusCode UA_STATUSCODE_BADPARENTNODEIDINVALID = 0x805b0000; // The parent node id does not to refer to a valid node.
static const UA_StatusCode UA_STATUSCODE_BADREFERENCENOTALLOWED = 0x805c0000; // The reference could not be created because it violates constraints imposed by the data model.
static const UA_StatusCode UA_STATUSCODE_BADNODEIDREJECTED = 0x805d0000; // The requested node id was reject because it was either invalid or server does not allow node ids to be specified by the client.
static const UA_StatusCode UA_STATUSCODE_BADNODEIDEXISTS = 0x805e0000; // The requested node id is already used by another node.
static const UA_StatusCode UA_STATUSCODE_BADNODECLASSINVALID = 0x805f0000; // The node class is not valid.
static const UA_StatusCode UA_STATUSCODE_BADBROWSENAMEINVALID = 0x80600000; // The browse name is invalid.
static const UA_StatusCode UA_STATUSCODE_BADBROWSENAMEDUPLICATED = 0x80610000; // The browse name is not unique among nodes that share the same relationship with the parent.
static const UA_StatusCode UA_STATUSCODE_BADNODEATTRIBUTESINVALID = 0x80620000; // The node attributes are not valid for the node class.
static const UA_StatusCode UA_STATUSCODE_BADTYPEDEFINITIONINVALID = 0x80630000; // The type definition node id does not reference an appropriate type node.
static const UA_StatusCode UA_STATUSCODE_BADSOURCENODEIDINVALID = 0x80640000; // The source node id does not reference a valid node.
static const UA_StatusCode UA_STATUSCODE_BADTARGETNODEIDINVALID = 0x80650000; // The target node id does not reference a valid node.
static const UA_StatusCode UA_STATUSCODE_BADDUPLICATEREFERENCENOTALLOWED = 0x80660000; // The reference type between the nodes is already defined.
static const UA_StatusCode UA_STATUSCODE_BADINVALIDSELFREFERENCE = 0x80670000; // The server does not allow this type of self reference on this node.
static const UA_StatusCode UA_STATUSCODE_BADREFERENCELOCALONLY = 0x80680000; // The reference type is not valid for a reference to a remote server.
static const UA_StatusCode UA_STATUSCODE_BADNODELETERIGHTS = 0x80690000; // The server will not allow the node to be deleted.
static const UA_StatusCode UA_STATUSCODE_UNCERTAINREFERENCENOTDELETED = 0x40bc0000; // The server was not able to delete all target references.
static const UA_StatusCode UA_STATUSCODE_BADSERVERINDEXINVALID = 0x806a0000; // The server index is not valid.
static const UA_StatusCode UA_STATUSCODE_BADVIEWIDUNKNOWN = 0x806b0000; // The view id does not refer to a valid view node.
static const UA_StatusCode UA_STATUSCODE_BADVIEWTIMESTAMPINVALID = 0x80c90000; // The view timestamp is not available or not supported.
static const UA_StatusCode UA_STATUSCODE_BADVIEWPARAMETERMISMATCH = 0x80ca0000; // The view parameters are not consistent with each other.
static const UA_StatusCode UA_STATUSCODE_BADVIEWVERSIONINVALID = 0x80cb0000; // The view version is not available or not supported.
static const UA_StatusCode UA_STATUSCODE_UNCERTAINNOTALLNODESAVAILABLE = 0x40c00000; // The list of references may not be complete because the underlying system is not available.
static const UA_StatusCode UA_STATUSCODE_GOODRESULTSMAYBEINCOMPLETE = 0x00ba0000; // The server should have followed a reference to a node in a remote server but did not. The result set may be incomplete.
static const UA_StatusCode UA_STATUSCODE_BADNOTTYPEDEFINITION = 0x80c80000; // The provided Nodeid was not a type definition nodeid.
static const UA_StatusCode UA_STATUSCODE_UNCERTAINREFERENCEOUTOFSERVER = 0x406c0000; // One of the references to follow in the relative path references to a node in the address space in another server.
static const UA_StatusCode UA_STATUSCODE_BADTOOMANYMATCHES = 0x806d0000; // The requested operation has too many matches to return.
static const UA_StatusCode UA_STATUSCODE_BADQUERYTOOCOMPLEX = 0x806e0000; // The requested operation requires too many resources in the server.
static const UA_StatusCode UA_STATUSCODE_BADNOMATCH = 0x806f0000; // The requested operation has no match to return.
static const UA_StatusCode UA_STATUSCODE_BADMAXAGEINVALID = 0x80700000; // The max age parameter is invalid.
static const UA_StatusCode UA_STATUSCODE_BADHISTORYOPERATIONINVALID = 0x80710000; // The history details parameter is not valid.
static const UA_StatusCode UA_STATUSCODE_BADHISTORYOPERATIONUNSUPPORTED = 0x80720000; // The server does not support the requested operation.
static const UA_StatusCode UA_STATUSCODE_BADINVALIDTIMESTAMPARGUMENT = 0x80bd0000; // The defined timestamp to return was invalid.
static const UA_StatusCode UA_STATUSCODE_BADWRITENOTSUPPORTED = 0x80730000; // The server not does support writing the combination of value, status and timestamps provided.
static const UA_StatusCode UA_STATUSCODE_BADTYPEMISMATCH = 0x80740000; // The value supplied for the attribute is not of the same type as the attribute's value.
static const UA_StatusCode UA_STATUSCODE_BADMETHODINVALID = 0x80750000; // The method id does not refer to a method for the specified object.
static const UA_StatusCode UA_STATUSCODE_BADARGUMENTSMISSING = 0x80760000; // The client did not specify all of the input arguments for the method.
static const UA_StatusCode UA_STATUSCODE_BADTOOMANYSUBSCRIPTIONS = 0x80770000; // The server has reached its  maximum number of subscriptions.
static const UA_StatusCode UA_STATUSCODE_BADTOOMANYPUBLISHREQUESTS = 0x80780000; // The server has reached the maximum number of queued publish requests.
static const UA_StatusCode UA_STATUSCODE_BADNOSUBSCRIPTION = 0x80790000; // There is no subscription available for this session.
static const UA_StatusCode UA_STATUSCODE_BADSEQUENCENUMBERUNKNOWN = 0x807a0000; // The sequence number is unknown to the server.
static const UA_StatusCode UA_STATUSCODE_BADMESSAGENOTAVAILABLE = 0x807b0000; // The requested notification message is no longer available.
static const UA_StatusCode UA_STATUSCODE_BADINSUFFICIENTCLIENTPROFILE = 0x807c0000; // The Client of the current Session does not support one or more Profiles that are necessary for the Subscription.
static const UA_StatusCode UA_STATUSCODE_BADSTATENOTACTIVE = 0x80bf0000; // The sub-state machine is not currently active.
static const UA_StatusCode UA_STATUSCODE_BADTCPSERVERTOOBUSY = 0x807d0000; // The server cannot process the request because it is too busy.
static const UA_StatusCode UA_STATUSCODE_BADTCPMESSAGETYPEINVALID = 0x807e0000; // The type of the message specified in the header invalid.
static const UA_StatusCode UA_STATUSCODE_BADTCPSECURECHANNELUNKNOWN = 0x807f0000; // The SecureChannelId and/or TokenId are not currently in use.
static const UA_StatusCode UA_STATUSCODE_BADTCPMESSAGETOOLARGE = 0x80800000; // The size of the message specified in the header is too large.
static const UA_StatusCode UA_STATUSCODE_BADTCPNOTENOUGHRESOURCES = 0x80810000; // There are not enough resources to process the request.
static const UA_StatusCode UA_STATUSCODE_BADTCPINTERNALERROR = 0x80820000; // An internal error occurred.
static const UA_StatusCode UA_STATUSCODE_BADTCPENDPOINTURLINVALID = 0x80830000; // The Server does not recognize the QueryString specified.
static const UA_StatusCode UA_STATUSCODE_BADREQUESTINTERRUPTED = 0x80840000; // The request could not be sent because of a network interruption.
static const UA_StatusCode UA_STATUSCODE_BADREQUESTTIMEOUT = 0x80850000; // Timeout occurred while processing the request.
static const UA_StatusCode UA_STATUSCODE_BADSECURECHANNELCLOSED = 0x80860000; // The secure channel has been closed.
static const UA_StatusCode UA_STATUSCODE_BADSECURECHANNELTOKENUNKNOWN = 0x80870000; // The token has expired or is not recognized.
static const UA_StatusCode UA_STATUSCODE_BADSEQUENCENUMBERINVALID = 0x80880000; // The sequence number is not valid.
static const UA_StatusCode UA_STATUSCODE_BADPROTOCOLVERSIONUNSUPPORTED = 0x80be0000; // The applications do not have compatible protocol versions.
static const UA_StatusCode UA_STATUSCODE_BADCONFIGURATIONERROR = 0x80890000; // There is a problem with the configuration that affects the usefulness of the value.
static const UA_StatusCode UA_STATUSCODE_BADNOTCONNECTED = 0x808a0000; // The variable should receive its value from another variable, but has never been configured to do so.
static const UA_StatusCode UA_STATUSCODE_BADDEVICEFAILURE = 0x808b0000; // There has been a failure in the device/data source that generates the value that has affected the value.
static const UA_StatusCode UA_STATUSCODE_BADSENSORFAILURE = 0x808c0000; // There has been a failure in the sensor from which the value is derived by the device/data source.
static const UA_StatusCode UA_STATUSCODE_BADOUTOFSERVICE = 0x808d0000; // The source of the data is not operational.
static const UA_StatusCode UA_STATUSCODE_BADDEADBANDFILTERINVALID = 0x808e0000; // The deadband filter is not valid.
static const UA_StatusCode UA_STATUSCODE_UNCERTAINNOCOMMUNICATIONLASTUSABLEVALUE = 0x408f0000; // Communication to the data source has failed. The variable value is the last value that had a good quality.
static const UA_StatusCode UA_STATUSCODE_UNCERTAINLASTUSABLEVALUE = 0x40900000; // Whatever was updating this value has stopped doing so.
static const UA_StatusCode UA_STATUSCODE_UNCERTAINSUBSTITUTEVALUE = 0x40910000; // The value is an operational value that was manually overwritten.
static const UA_StatusCode UA_STATUSCODE_UNCERTAININITIALVALUE = 0x40920000; // The value is an initial value for a variable that normally receives its value from another variable.
static const UA_StatusCode UA_STATUSCODE_UNCERTAINSENSORNOTACCURATE = 0x40930000; // The value is at one of the sensor limits.
static const UA_StatusCode UA_STATUSCODE_UNCERTAINENGINEERINGUNITSEXCEEDED = 0x40940000; // The value is outside of the range of values defined for this parameter.
static const UA_StatusCode UA_STATUSCODE_UNCERTAINSUBNORMAL = 0x40950000; // The value is derived from multiple sources and has less than the required number of Good sources.
static const UA_StatusCode UA_STATUSCODE_GOODLOCALOVERRIDE = 0x00960000; // The value has been overridden.
static const UA_StatusCode UA_STATUSCODE_BADREFRESHINPROGRESS = 0x80970000; // This Condition refresh failed, a Condition refresh operation is already in progress.
static const UA_StatusCode UA_STATUSCODE_BADCONDITIONALREADYDISABLED = 0x80980000; // This condition has already been disabled.
static const UA_StatusCode UA_STATUSCODE_BADCONDITIONALREADYENABLED = 0x80cc0000; // This condition has already been enabled.
static const UA_StatusCode UA_STATUSCODE_BADCONDITIONDISABLED = 0x80990000; // Property not available, this condition is disabled.
static const UA_StatusCode UA_STATUSCODE_BADEVENTIDUNKNOWN = 0x809a0000; // The specified event id is not recognized.
static const UA_StatusCode UA_STATUSCODE_BADEVENTNOTACKNOWLEDGEABLE = 0x80bb0000; // The event cannot be acknowledged.
static const UA_StatusCode UA_STATUSCODE_BADDIALOGNOTACTIVE = 0x80cd0000; // The dialog condition is not active.
static const UA_StatusCode UA_STATUSCODE_BADDIALOGRESPONSEINVALID = 0x80ce0000; // The response is not valid for the dialog.
static const UA_StatusCode UA_STATUSCODE_BADCONDITIONBRANCHALREADYACKED = 0x80cf0000; // The condition branch has already been acknowledged.
static const UA_StatusCode UA_STATUSCODE_BADCONDITIONBRANCHALREADYCONFIRMED = 0x80d00000; // The condition branch has already been confirmed.
static const UA_StatusCode UA_STATUSCODE_BADCONDITIONALREADYSHELVED = 0x80d10000; // The condition has already been shelved.
static const UA_StatusCode UA_STATUSCODE_BADCONDITIONNOTSHELVED = 0x80d20000; // The condition is not currently shelved.
static const UA_StatusCode UA_STATUSCODE_BADSHELVINGTIMEOUTOFRANGE = 0x80d30000; // The shelving time not within an acceptable range.
static const UA_StatusCode UA_STATUSCODE_BADNODATA = 0x809b0000; // No data exists for the requested time range or event filter.
static const UA_StatusCode UA_STATUSCODE_BADBOUNDNOTFOUND = 0x80d70000; // No data found to provide upper or lower bound value.
static const UA_StatusCode UA_STATUSCODE_BADBOUNDNOTSUPPORTED = 0x80d80000; // The server cannot retrieve a bound for the variable.
static const UA_StatusCode UA_STATUSCODE_BADDATALOST = 0x809d0000; // Data is missing due to collection started/stopped/lost.
static const UA_StatusCode UA_STATUSCODE_BADDATAUNAVAILABLE = 0x809e0000; // Expected data is unavailable for the requested time range due to an un-mounted volume, an off-line archive or tape, or similar reason for temporary unavailability.
static const UA_StatusCode UA_STATUSCODE_BADENTRYEXISTS = 0x809f0000; // The data or event was not successfully inserted because a matching entry exists.
static const UA_StatusCode UA_STATUSCODE_BADNOENTRYEXISTS = 0x80a00000; // The data or event was not successfully updated because no matching entry exists.
static const UA_StatusCode UA_STATUSCODE_BADTIMESTAMPNOTSUPPORTED = 0x80a10000; // The client requested history using a timestamp format the server does not support (i.e requested ServerTimestamp when server only supports SourceTimestamp).
static const UA_StatusCode UA_STATUSCODE_GOODENTRYINSERTED = 0x00a20000; // The data or event was successfully inserted into the historical database.
static const UA_StatusCode UA_STATUSCODE_GOODENTRYREPLACED = 0x00a30000; // The data or event field was successfully replaced in the historical database.
static const UA_StatusCode UA_STATUSCODE_UNCERTAINDATASUBNORMAL = 0x40a40000; // The value is derived from multiple values and has less than the required number of Good values.
static const UA_StatusCode UA_STATUSCODE_GOODNODATA = 0x00a50000; // No data exists for the requested time range or event filter.
static const UA_StatusCode UA_STATUSCODE_GOODMOREDATA = 0x00a60000; // The data or event field was successfully replaced in the historical database.
static const UA_StatusCode UA_STATUSCODE_BADAGGREGATELISTMISMATCH = 0x80d40000; // The requested number of Aggregates does not match the requested number of NodeIds.
static const UA_StatusCode UA_STATUSCODE_BADAGGREGATENOTSUPPORTED = 0x80d50000; // The requested Aggregate is not support by the server.
static const UA_StatusCode UA_STATUSCODE_BADAGGREGATEINVALIDINPUTS = 0x80d60000; // The aggregate value could not be derived due to invalid data inputs.
static const UA_StatusCode UA_STATUSCODE_BADAGGREGATECONFIGURATIONREJECTED = 0x80da0000; // The aggregate configuration is not valid for specified node.
static const UA_StatusCode UA_STATUSCODE_GOODDATAIGNORED = 0x00d90000; // The request pecifies fields which are not valid for the EventType or cannot be saved by the historian.
static const UA_StatusCode UA_STATUSCODE_GOODCOMMUNICATIONEVENT = 0x00a70000; // The communication layer has raised an event.
static const UA_StatusCode UA_STATUSCODE_GOODSHUTDOWNEVENT = 0x00a80000; // The system is shutting down.
static const UA_StatusCode UA_STATUSCODE_GOODCALLAGAIN = 0x00a90000; // The operation is not finished and needs to be called again.
static const UA_StatusCode UA_STATUSCODE_GOODNONCRITICALTIMEOUT = 0x00aa0000; // A non-critical timeout occurred.
static const UA_StatusCode UA_STATUSCODE_BADINVALIDARGUMENT = 0x80ab0000; // One or more arguments are invalid.
static const UA_StatusCode UA_STATUSCODE_BADCONNECTIONREJECTED = 0x80ac0000; // Could not establish a network connection to remote server.
static const UA_StatusCode UA_STATUSCODE_BADDISCONNECT = 0x80ad0000; // The server has disconnected from the client.
static const UA_StatusCode UA_STATUSCODE_BADCONNECTIONCLOSED = 0x80ae0000; // The network connection has been closed.
static const UA_StatusCode UA_STATUSCODE_BADINVALIDSTATE = 0x80af0000; // The operation cannot be completed because the object is closed, uninitialized or in some other invalid state.
static const UA_StatusCode UA_STATUSCODE_BADENDOFSTREAM = 0x80b00000; // Cannot move beyond end of the stream.
static const UA_StatusCode UA_STATUSCODE_BADNODATAAVAILABLE = 0x80b10000; // No data is currently available for reading from a non-blocking stream.
static const UA_StatusCode UA_STATUSCODE_BADWAITINGFORRESPONSE = 0x80b20000; // The asynchronous operation is waiting for a response.
static const UA_StatusCode UA_STATUSCODE_BADOPERATIONABANDONED = 0x80b30000; // The asynchronous operation was abandoned by the caller.
static const UA_StatusCode UA_STATUSCODE_BADEXPECTEDSTREAMTOBLOCK = 0x80b40000; // The stream did not return all data requested (possibly because it is a non-blocking stream).
static const UA_StatusCode UA_STATUSCODE_BADWOULDBLOCK = 0x80b50000; // Non blocking behaviour is required and the operation would block.
static const UA_StatusCode UA_STATUSCODE_BADSYNTAXERROR = 0x80b60000; // A value had an invalid syntax.
static const UA_StatusCode UA_STATUSCODE_BADMAXCONNECTIONSREACHED = 0x80b70000; // The operation could not be finished because all available connections are in use.

#endif /* OPCUA_STATUSCODES_H_ */
